#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "ADT10.h"

/*{{{  conversion functions */

ATerm ADT10_stringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

char *ADT10_charsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _ADT10_Entries;
typedef struct ATerm _ADT10_Entry;

/*}}}  */

/*{{{  void ADT10_initADT10Api(void) */

void ADT10_initADT10Api(void)
{
  init_ADT10_dict();
}

/*}}}  */

/*{{{  protect functions */

void ADT10_protectEntries(ADT10_Entries *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT10_protectEntry(ADT10_Entry *arg)
{
  ATprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  ADT10_Entries ADT10_EntriesFromTerm(ATerm t) */

ADT10_Entries ADT10_EntriesFromTerm(ATerm t)
{
  return (ADT10_Entries)t;
}

/*}}}  */
/*{{{  ATerm ADT10_EntriesToTerm(ADT10_Entries arg) */

ATerm ADT10_EntriesToTerm(ADT10_Entries arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT10_Entry ADT10_EntryFromTerm(ATerm t) */

ADT10_Entry ADT10_EntryFromTerm(ATerm t)
{
  return (ADT10_Entry)t;
}

/*}}}  */
/*{{{  ATerm ADT10_EntryToTerm(ADT10_Entry arg) */

ATerm ADT10_EntryToTerm(ADT10_Entry arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */


/*}}}  */
/*{{{  constructors */

/*{{{  ADT10_Entries ADT10_makeEntriesEmpty(void) */

ADT10_Entries ADT10_makeEntriesEmpty(void)
{
  return (ADT10_Entries)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT10_Entries ADT10_makeEntriesList(ADT10_Entry head, ADT10_Entries tail) */

ADT10_Entries ADT10_makeEntriesList(ADT10_Entry head, ADT10_Entries tail)
{
  return (ADT10_Entries)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT10_Entry ADT10_makeEntryConstructor(ATerm sort, ATerm alternative, ATerm termPattern) */

ADT10_Entry ADT10_makeEntryConstructor(ATerm sort, ATerm alternative, ATerm termPattern)
{
  return (ADT10_Entry)(ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) termPattern), (ATerm) alternative), (ATerm) sort);
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool ADT10_isEqualEntries(ADT10_Entries arg0, ADT10_Entries arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT10_isEqualEntry(ADT10_Entry arg0, ADT10_Entry arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  ADT10_Entries accessors */

/*{{{  ATbool ADT10_isValidEntries(ADT10_Entries arg) */

ATbool ADT10_isValidEntries(ADT10_Entries arg)
{
  if (ADT10_isEntriesEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT10_isEntriesList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT10_isEntriesEmpty(ADT10_Entries arg) */

inline ATbool ADT10_isEntriesEmpty(ADT10_Entries arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT10_patternEntriesEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT10_isEntriesList(ADT10_Entries arg) */

inline ATbool ADT10_isEntriesList(ADT10_Entries arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT10_patternEntriesList, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADT10_hasEntriesHead(ADT10_Entries arg) */

ATbool ADT10_hasEntriesHead(ADT10_Entries arg)
{
  if (ADT10_isEntriesList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT10_hasEntriesTail(ADT10_Entries arg) */

ATbool ADT10_hasEntriesTail(ADT10_Entries arg)
{
  if (ADT10_isEntriesList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT10_Entry ADT10_getEntriesHead(ADT10_Entries arg) */

ADT10_Entry ADT10_getEntriesHead(ADT10_Entries arg)
{
  
    return (ADT10_Entry)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT10_Entries ADT10_getEntriesTail(ADT10_Entries arg) */

ADT10_Entries ADT10_getEntriesTail(ADT10_Entries arg)
{
  
    return (ADT10_Entries)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT10_Entries ADT10_setEntriesHead(ADT10_Entries arg, ADT10_Entry head) */

ADT10_Entries ADT10_setEntriesHead(ADT10_Entries arg, ADT10_Entry head)
{
  if (ADT10_isEntriesList(arg)) {
    return (ADT10_Entries)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Entries has no Head: %t\n", arg);
  return (ADT10_Entries)NULL;
}

/*}}}  */
/*{{{  ADT10_Entries ADT10_setEntriesTail(ADT10_Entries arg, ADT10_Entries tail) */

ADT10_Entries ADT10_setEntriesTail(ADT10_Entries arg, ADT10_Entries tail)
{
  if (ADT10_isEntriesList(arg)) {
    return (ADT10_Entries)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Entries has no Tail: %t\n", arg);
  return (ADT10_Entries)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT10_Entry accessors */

/*{{{  ATbool ADT10_isValidEntry(ADT10_Entry arg) */

ATbool ADT10_isValidEntry(ADT10_Entry arg)
{
  if (ADT10_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT10_isEntryConstructor(ADT10_Entry arg) */

inline ATbool ADT10_isEntryConstructor(ADT10_Entry arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT10_patternEntryConstructor, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADT10_hasEntrySort(ADT10_Entry arg) */

ATbool ADT10_hasEntrySort(ADT10_Entry arg)
{
  if (ADT10_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT10_hasEntryAlternative(ADT10_Entry arg) */

ATbool ADT10_hasEntryAlternative(ADT10_Entry arg)
{
  if (ADT10_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT10_hasEntryTermPattern(ADT10_Entry arg) */

ATbool ADT10_hasEntryTermPattern(ADT10_Entry arg)
{
  if (ADT10_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATerm ADT10_getEntrySort(ADT10_Entry arg) */

ATerm ADT10_getEntrySort(ADT10_Entry arg)
{
  
    return (ATerm)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ATerm ADT10_getEntryAlternative(ADT10_Entry arg) */

ATerm ADT10_getEntryAlternative(ADT10_Entry arg)
{
  
    return (ATerm)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  ATerm ADT10_getEntryTermPattern(ADT10_Entry arg) */

ATerm ADT10_getEntryTermPattern(ADT10_Entry arg)
{
  
    return (ATerm)ATelementAt((ATermList)arg, 2);
}

/*}}}  */
/*{{{  ADT10_Entry ADT10_setEntrySort(ADT10_Entry arg, ATerm sort) */

ADT10_Entry ADT10_setEntrySort(ADT10_Entry arg, ATerm sort)
{
  if (ADT10_isEntryConstructor(arg)) {
    return (ADT10_Entry)ATreplace((ATermList)arg, (ATerm)((ATerm) sort), 0);
  }

  ATabort("Entry has no Sort: %t\n", arg);
  return (ADT10_Entry)NULL;
}

/*}}}  */
/*{{{  ADT10_Entry ADT10_setEntryAlternative(ADT10_Entry arg, ATerm alternative) */

ADT10_Entry ADT10_setEntryAlternative(ADT10_Entry arg, ATerm alternative)
{
  if (ADT10_isEntryConstructor(arg)) {
    return (ADT10_Entry)ATreplace((ATermList)arg, (ATerm)((ATerm) alternative), 1);
  }

  ATabort("Entry has no Alternative: %t\n", arg);
  return (ADT10_Entry)NULL;
}

/*}}}  */
/*{{{  ADT10_Entry ADT10_setEntryTermPattern(ADT10_Entry arg, ATerm termPattern) */

ADT10_Entry ADT10_setEntryTermPattern(ADT10_Entry arg, ATerm termPattern)
{
  if (ADT10_isEntryConstructor(arg)) {
    return (ADT10_Entry)ATreplace((ATermList)arg, (ATerm)((ATerm) termPattern), 2);
  }

  ATabort("Entry has no TermPattern: %t\n", arg);
  return (ADT10_Entry)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  ADT10_Entries ADT10_visitEntries(ADT10_Entries arg, ADT10_Entry (*acceptHead)(ADT10_Entry)) */

ADT10_Entries ADT10_visitEntries(ADT10_Entries arg, ADT10_Entry (*acceptHead)(ADT10_Entry))
{
  if (ADT10_isEntriesEmpty(arg)) {
    return ADT10_makeEntriesEmpty();
  }
  if (ADT10_isEntriesList(arg)) {
    return ADT10_makeEntriesList(
        acceptHead ? acceptHead(ADT10_getEntriesHead(arg)) : ADT10_getEntriesHead(arg),
        ADT10_visitEntries(ADT10_getEntriesTail(arg), acceptHead));
  }
  ATabort("not a Entries: %t\n", arg);
  return (ADT10_Entries)NULL;
}

/*}}}  */
/*{{{  ADT10_Entry ADT10_visitEntry(ADT10_Entry arg, ATerm (*acceptSort)(ATerm), ATerm (*acceptAlternative)(ATerm), ATerm (*acceptTermPattern)(ATerm)) */

ADT10_Entry ADT10_visitEntry(ADT10_Entry arg, ATerm (*acceptSort)(ATerm), ATerm (*acceptAlternative)(ATerm), ATerm (*acceptTermPattern)(ATerm))
{
  if (ADT10_isEntryConstructor(arg)) {
    return ADT10_makeEntryConstructor(
        acceptSort ? acceptSort(ADT10_getEntrySort(arg)) : ADT10_getEntrySort(arg),
        acceptAlternative ? acceptAlternative(ADT10_getEntryAlternative(arg)) : ADT10_getEntryAlternative(arg),
        acceptTermPattern ? acceptTermPattern(ADT10_getEntryTermPattern(arg)) : ADT10_getEntryTermPattern(arg));
  }
  ATabort("not a Entry: %t\n", arg);
  return (ADT10_Entry)NULL;
}

/*}}}  */

/*}}}  */
